﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using NAudio;
using NAudio.Midi;
using MidiStylusLibrary;

namespace MidiStylus
{
    public partial class SettingsForm : Form
    {
        #region Variables

        private Configuration _configuration = new Configuration();

        #endregion

        #region Constructors

        public SettingsForm()
        {
            InitializeComponent();

            // Initialize the comboboxes with default values
            InitializeControls();

            // Populate the settings from XML
            LoadSettings();
        }

        #endregion

        #region Methods

        private void InitializeControls()
        {
            try
            {
                cbMIDIDevices.Items.Clear();
                cbPressureValueControl.Items.Clear();
                cbXValueControl.Items.Clear();
                cbYValueControl.Items.Clear();
                cbMIDIChannel.Items.Clear();

                // Initialize the comboboxes
                for (int i = 0; i < 128; i++)
                {
                    cbPressureValueControl.Items.Add(i);
                    cbXValueControl.Items.Add(i);
                    cbYValueControl.Items.Add(i);
                }

                for (int i = 0; i < 16; i++)
                {
                    cbMIDIChannel.Items.Add(i);
                }

                // Get the product name for each device found 
                for (int device = 0; device < MidiOut.NumberOfDevices; device++)
                {
                    cbMIDIDevices.Items.Add(MidiOut.DeviceInfo(device).ProductName);
                }

                if (cbMIDIDevices.Items.Count > 0) cbMIDIDevices.SelectedIndex = 0;

                cbPressureValueControl.SelectedIndex = 0;
                cbXValueControl.SelectedIndex = 0;
                cbYValueControl.SelectedIndex = 0;
                cbMIDIChannel.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void LoadSettings()
        {
            // Load configuration data from XML
            try
            {
                _configuration = new Configuration("Settings.xml");

                cbMIDIDevices.SelectedIndex = _configuration.DeviceNumber;
                cbPressureValueControl.SelectedIndex = _configuration.PressureControlNumber;
                cbXValueControl.SelectedIndex = _configuration.XControlNumber;
                cbYValueControl.SelectedIndex = _configuration.YControlNumber;
                cbMIDIChannel.SelectedIndex = _configuration.ChannelNumber;
            }
            catch (Exception ex)
            {
                MessageBox.Show("No configuration file found, using default settings", "No configuration file", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void SaveSettings()
        {
            try
            {
                _configuration.PressureControlNumber = cbPressureValueControl.SelectedIndex.ToInteger32();
                _configuration.XControlNumber = cbXValueControl.SelectedIndex.ToInteger32();
                _configuration.YControlNumber = cbYValueControl.SelectedIndex.ToInteger32();
                _configuration.DeviceNumber = cbMIDIDevices.SelectedIndex.ToInteger32();
                _configuration.ChannelNumber = cbMIDIChannel.SelectedIndex.ToInteger32();

                _configuration.SaveConfiguration("Settings.xml");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void Test(int controlNumber)
        {
            this.SaveSettings();
            ((MainForm)this.Owner).InitializeMIDI();

            if (((MainForm)this.Owner).IsReady)
            {
                ((MainForm)this.Owner).MidiInterface.ControlChange(_configuration.ChannelNumber, controlNumber, 64);
                MessageBox.Show("Value 64 sent to control " + controlNumber + " on channel " + _configuration.ChannelNumber);
            }
        }

        #endregion

        #region Event Handlers

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            this.SaveSettings();
            this.DialogResult = DialogResult.OK;
        }

        private void btnTestY_Click(object sender, EventArgs e)
        {
            this.Test(this.cbYValueControl.SelectedIndex);
        }

        private void btnTestPressure_Click(object sender, EventArgs e)
        {
            this.Test(this.cbPressureValueControl.SelectedIndex);
        }

        private void btnTestX_Click(object sender, EventArgs e)
        {
            this.Test(this.cbXValueControl.SelectedIndex);
        }

        #endregion
    }
}
