﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace MidiStylusLibrary
{
    public class Configuration
    {
        #region Properties

        public int DeviceNumber { get; set; }
        public int XControlNumber { get; set; }
        public int YControlNumber { get; set; }
        public int PressureControlNumber { get; set; }
        public int ChannelNumber { get; set; }

        #endregion

        #region Constructors

        public Configuration()
        {
        }

        public Configuration(string filePath)
        {
            this.DeviceNumber = 0;
            this.PressureControlNumber = 0;
            this.XControlNumber = 0;
            this.YControlNumber = 0;

            this.LoadConfiguration(filePath);
        }

        #endregion

        #region Methods

        public void SaveConfiguration(string filePath)
        {
            try
            {
                XElement configuration =
                    new XElement("MidiStylusSettings",
                        new XElement("MIDIChannel", this.ChannelNumber.ToString()),
                        new XElement("DeviceNumber", this.DeviceNumber.ToString()),
                        new XElement("XControlNumber", this.XControlNumber.ToString()),
                        new XElement("YControlNumber", this.YControlNumber.ToString()),
                        new XElement("PressureControlNumber", this.PressureControlNumber.ToString())
                        );

                configuration.Save(filePath);
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        public void LoadConfiguration(string filePath)
        {
            try
            {
                // Parse the configuration XML file
                XElement configuration = XElement.Load(filePath);

                this.DeviceNumber = ((int?)configuration.Element("DeviceNumber") ?? 0).ToInteger32();
                this.XControlNumber = ((int?)configuration.Element("XControlNumber") ?? 0).ToInteger32();
                this.YControlNumber = ((int?)configuration.Element("YControlNumber") ?? 0).ToInteger32();
                this.PressureControlNumber = ((int?)configuration.Element("PressureControlNumber") ?? 0).ToInteger32();
                this.ChannelNumber = ((int?)configuration.Element("ChannelNumber") ?? 0).ToInteger32();
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        #endregion
    }
}
