﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.StylusInput;
using Microsoft.StylusInput.PluginData;
using System.Windows.Forms;
using System.Drawing;

namespace MidiStylusLibrary
{
    public class StylusData : IStylusSyncPlugin
    {
        #region Variables

        private Control attachedControl;

        #endregion

        #region Properties

        public int Pressure { get; set; }

        public int XValue { get; set; }
        
        public int YValue { get; set; }

        #endregion

        #region Constructors

        public StylusData(Control control)
        {
            this.attachedControl = control;
        }

        #endregion

        #region Methods

        public static int NormalizeValue(int input, int maxInputValue, int maxOutputValue)
        {
            int result = 0;
            
            if (maxInputValue > 0)
                result = (input * maxOutputValue) / maxInputValue;

            return result;
        }

        public DataInterestMask DataInterest
        {
            get
            {
                return DataInterestMask.Packets |
                    DataInterestMask.StylusDown |
                    DataInterestMask.StylusUp;
            }
        }

        public void Packets(RealTimeStylus sender, PacketsData data)
        {
            Console.WriteLine("Number of Packets: " +
                data.Count.ToString());

            Console.WriteLine("Properties per packet: " +
                data.PacketPropertyCount.ToString());

            Graphics g = this.attachedControl.CreateGraphics();

            int packetCounter;

            for (packetCounter = 0;
                packetCounter <= (data.Count - data.PacketPropertyCount);
                packetCounter += data.PacketPropertyCount)
                {
                this.XValue = (int)(g.DpiX * data[packetCounter] / 2540);
                this.YValue = (int)(g.DpiY * data[packetCounter + 1] / 2540);

                if (data.PacketPropertyCount > 2)
                {
                    this.Pressure = data[packetCounter + 2];
                }
            }
        }

        public void StylusDown(RealTimeStylus sender, StylusDownData data)
        {
            Graphics g = this.attachedControl.CreateGraphics();

            int packetCounter;

            for (packetCounter = 0;
                packetCounter <= (data.Count - data.PacketPropertyCount);
                packetCounter += data.PacketPropertyCount)
            {
                this.XValue = (int)(g.DpiX * data[packetCounter] / 2540);
                this.YValue = (int)(g.DpiY * data[packetCounter + 1] / 2540);

                if (data.PacketPropertyCount > 2)
                {
                    this.Pressure = data[packetCounter + 2];
                }
            }
        }

        public void StylusUp(RealTimeStylus sender, StylusUpData data)
        {
            // Handler stylus up event
        }

        public void TabletRemoved(RealTimeStylus sender, TabletRemovedData data)
        {
        }

        public void CustomStylusDataAdded(RealTimeStylus sender, CustomStylusData data)
        {
        }

        public void RealTimeStylusDisabled(RealTimeStylus sender, RealTimeStylusDisabledData data)
        {
        }

        public void StylusOutOfRange(RealTimeStylus sender, StylusOutOfRangeData data)
        {
        }

        public void StylusButtonUp(RealTimeStylus sender, StylusButtonUpData data)
        {
        }

        public void StylusInRange(RealTimeStylus sender, StylusInRangeData data)
        {
        }

        public void RealTimeStylusEnabled(RealTimeStylus sender, RealTimeStylusEnabledData data)
        {
        }

        public void SystemGesture(RealTimeStylus sender, SystemGestureData data)
        {
        }

        public void TabletAdded(RealTimeStylus sender, TabletAddedData data)
        {
        }

        public void Error(RealTimeStylus sender, ErrorData data)
        {
        }

        public void InAirPackets(RealTimeStylus sender, InAirPacketsData data)
        {
        }

        public void StylusButtonDown(RealTimeStylus sender, StylusButtonDownData data)
        {
        }

        #endregion
    }
}
